@@ -28,7 +28,7 @@
 
 import unittest
 
-from sql import Table, Literal
+from sql import Table, Literal, With
 
 
 class TestUpdate(unittest.TestCase):
@@ -70,3 +70,47 @@ class TestUpdate(unittest.TestCase):
         self.assertEqual(str(query),
             'UPDATE "t" SET "c" = %s RETURNING "t"."c"')
         self.assertEqual(query.params, ('foo',))
+
+    def test_update_returning_select(self):
+        t1 = Table('t1')
+        t2 = Table('t2')
+        query = t1.update([t1.c], ['foo'],
+            returning=[
+                t2.select(t2.c, where=(t2.c1 == t1.c) & (t2.c2 == 'bar'))])
+        self.assertEqual(str(query),
+            'UPDATE "t1" SET "c" = %s '
+            'RETURNING (SELECT "b"."c" FROM "t2" AS "b" '
+            'WHERE (("b"."c1" = "t1"."c") AND ("b"."c2" = %s)))')
+        self.assertEqual(query.params, ('foo', 'bar'))
+
+    def test_with(self):
+        t1 = Table('t1')
+        w = With(query=t1.select(t1.c1))
+
+        query = self.table.update(
+            [self.table.c2],
+            with_=[w],
+            values=[w.select(w.c3, where=w.c4 == 2)])
+        self.assertEqual(str(query),
+            'WITH "b" AS (SELECT "c"."c1" FROM "t1" AS "c") '
+            'UPDATE "t" SET "c2" = (SELECT "b"."c3" FROM "b" AS "b" '
+            'WHERE ("b"."c4" = %s))')
+        self.assertEqual(query.params, (2,))
+
+    def test_schema(self):
+        t1 = Table('t1', 'default')
+        query = t1.update([t1.c1], ['foo'])
+
+        self.assertEqual(str(query), 'UPDATE "default"."t1" SET "c1" = %s')
+        self.assertEqual(query.params, ('foo',))
+
+    def test_schema_subselect(self):
+        t1 = Table('t1', 'default')
+        t2 = Table('t2', 'default')
+        query = t1.update([t1.c1], t2.select(t2.c, where=t2.i == t1.i))
+
+        self.assertEqual(str(query),
+            'UPDATE "default"."t1" SET "c1" = ('
+            'SELECT "b"."c" FROM "default"."t2" AS "b" '
+            'WHERE ("b"."i" = "default"."t1"."i"))')
+        self.assertEqual(query.params, ())
